###############   PSQF 6271 Longitudinal MLM Example 3b using R   #############

# Set width of output and  number of digits to be printed
options("width"=120, "digits"=8) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs with link functions and R cov structures

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do linear combinations in gls

if (!require("emmeans")) install.packages("emmeans")
library(emmeans) # To get model-implied means in gls


################################################################################
#####               BEGIN EXAMPLE 3b DATA MANIUPLATION                     #####
################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6271/PSQF6271_Example3b/"
filename = "SAS_Chapter3b.sas7bdat"
setwd(dir=filesave)

# Import chapter 3 six-occasion stacked data with labels
Example3b = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame as data frame without labels to use for analysis
Example3b = as.data.frame(Example3b)

# Make new variable for session with reference=6 to match other programs
Example3b$session6=relevel(factor(Example3b$session), ref=6)

# Sort data by PersonID and session (needed for correct RCOV matrix)
Example3b = sort_asc(Example3b,PersonID,session)


################################################################################
#####                   BEGIN EXAMPLE 3b MODELS                            #####
################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example3b_Output.txt"))

print("Chapter 3b Example: Means by session for RT outcome")
describeBy(x=Example3b$rt, digits=2, group=Example3b$session)  

txtComment(" ") # insert blank space in output


print("Eq 3.10: Between-Person Independent ANOVA via R GLS (VC R matrix, =LM)")
BP = gls(data=Example3b, method="REML", model=rt~1+factor(session6), 
               correlation=NULL) # VC R matrix (no correlation)
print("Show results with -2LL, total leftover variance, and 95% CIs")
summary(BP); -2*logLik(BP); summary(BP)$sigma^2; confint(BP, level=.95)        
print("Session means, pairwise mean differences, and and omnibus F-test")
emmeans(ref_grid(BP), pairwise~(session6), adjust="none"); anova(BP) 

txtComment(" ") # insert blank space in output


print("Eq 3.10: Univariate Repeated Measures ANOVA via R GLS (CS R matrix)")
UWP = gls(data=Example3b, method="REML", model=rt~1+factor(session6), 
          correlation=corCompSymm(form=~1|PersonID)) 
print("Show results with -2LL, total leftover variance, and 95% CIs")
summary(UWP); -2*logLik(UWP); summary(UWP)$sigma^2; confint(UWP, level=.95) 

print("Show R and RCORR matrices for first person in data")
getVarCov(UWP, individual="101"); corMatrix(UWP$modelStruct$corStruct)[[5]]   

print("Session means, pairwise mean differences, and omnibus F-test")
emmeans(ref_grid(UWP), pairwise~session6, adjust="none"); anova(UWP)

print("Show likelihood ratio test comparing model fit: BP vs. Univ WP")
anova(UWP,BP)  # Order of "more", "fewer"

txtComment(" ") # insert blank space in output


print("Eq 3.10: Multivariate Repeated Measures ANOVA via R GLS (UN R matrix)")
MWP = gls(data=Example3b, method="REML",model=rt~1+factor(session6), 
          correlation=corSymm(form=~as.numeric(session)|PersonID), # UN correlations
          weights=varIdent(form=~1|session))                       # Het variances by session    
print("Show results with -2LL, total leftover variance, and 95% CIs")
summary(MWP); -2*logLik(MWP); summary(MWP)$sigma^2; confint(MWP, level=.95) 

print("Show R and RCORR matrices for first person in the data") 
getVarCov(MWP, individual="101"); corMatrix(MWP$modelStruct$corStruct)[[5]] 

print("Session means, pairwise mean differences, omnibus F-test")
emmeans(ref_grid(MWP), pairwise~session6, adjust="none") # tried mode="df.error"
print("Error because of Satterthwaite DDF, so had to switch to incorrect residual DDF")
lsmeans(MWP, "session6", mode="df.error"); joint_tests(MWP, mode="df.error")

print("Show likelihood ratio test comparing model fit: Univ WP vs. Multiv WP")
anova(MWP,UWP)  

txtComment(" ") # insert blank space in output


# Close external file of saved results
txtStop()
